package com.marvinelsen.chinese.phonetics

import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.PinyinMarkSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.PinyinNumberSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.SyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.ZhuyinSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.parsing.PinyinNumberSyllableParser
import com.marvinelsen.chinese.phonetics.internal.syllable.parsing.SyllableParser
import com.marvinelsen.chinese.phonetics.internal.syllable.parsing.ZhuyinSyllableParser

object ChinesePhonetics {
    fun getToneFromNumberedPinyin(numberedPinyin: String) =
        PinyinNumberSyllableParser.parseOrNull(numberedPinyin)?.tone

    fun getToneFromZhuyin(zhuyin: String) =
        ZhuyinSyllableParser.parseOrNull(zhuyin)?.tone

    fun getTonesFromNumberedPinyinSequence(numberedPinyinSequence: String) = numberedPinyinSequence.trim()
        .split(Regex("\\s+"))
        .filter { it.isNotEmpty() }
        .map { getToneFromNumberedPinyin(it) }

    fun getTonesFromZhuyinSequence(zhuyinSequence: String) = zhuyinSequence.trim()
        .split(Regex("\\s+"))
        .filter { it.isNotEmpty() }
        .map { getToneFromZhuyin(it) }

    fun zhuyinToPinyinWithNumbers(zhuyin: String, strict: Boolean = true) = convertSyllableSequence(
        input = zhuyin,
        parser = ZhuyinSyllableParser,
        formatter = PinyinNumberSyllableFormatter,
        strict = strict
    )

    fun zhuyinToPinyinWithToneMarks(zhuyin: String, strict: Boolean = true) = convertSyllableSequence(
        input = zhuyin,
        parser = ZhuyinSyllableParser,
        formatter = PinyinMarkSyllableFormatter,
        strict = strict
    )

    fun pinyinWithNumbersToZhuyin(pinyinWithNumbers: String, strict: Boolean = true) = convertSyllableSequence(
        input = pinyinWithNumbers,
        parser = PinyinNumberSyllableParser,
        formatter = ZhuyinSyllableFormatter,
        strict = strict
    )

    fun pinyinWithNumbersToToneMarks(pinyinWithNumbers: String, strict: Boolean = true) = convertSyllableSequence(
        input = pinyinWithNumbers,
        parser = PinyinNumberSyllableParser,
        formatter = PinyinMarkSyllableFormatter,
        strict = strict
    )

    private fun convertSyllableSequence(
        input: String,
        parser: SyllableParser,
        formatter: SyllableFormatter,
        strict: Boolean
    ): String {
        val originalParts = input.trim().split(Regex("\\s+")).filter { it.isNotEmpty() }
        if (originalParts.isEmpty()) return ""

        val convertedParts = originalParts.map { part ->
            if (strict) {
                val syllable = parser.parse(part)
                formatter.format(syllable)
            } else {
                val syllable = parser.parseOrNull(part)
                syllable?.let { formatter.format(it) } ?: part
            }
        }
        return convertedParts.joinToString(" ")
    }
}
