package com.marvinelsen.chinese.phonetics.internal.syllable

import java.io.IOException

/**
 * Manages the loading and access of Pinyin-Zhuyin transcription data.
 * Intended for internal use by parsers and formatters.
 * Loads data lazily upon first access.
 */
internal object TranscriptionDataRepository {
    private const val TRANSCRIPTION_RESOURCE_PATH = "/pinyin_zhuyin_transcriptions.tsv"

    val pinyinToZhuyin = loadTranscriptions()
    val zhuyinToPinyin = pinyinToZhuyin.entries.associate { it.value to it.key }

    fun isValidZhuyin(zhuyin: String) = zhuyin in zhuyinToPinyin
    fun isValidPinyin(pinyin: String) = pinyin in pinyinToZhuyin

    fun normalize(pinyin: String) = pinyin.lowercase()
        .replace("v", "ü")
        .replace("u:", "ü")

    private fun loadTranscriptions(): Map<String, String> {
        val inputStream = this::class.java.getResourceAsStream(TRANSCRIPTION_RESOURCE_PATH)
            ?: error("Cannot find transcription resource: $TRANSCRIPTION_RESOURCE_PATH")

        return try {
            inputStream.bufferedReader().useLines { lines ->
                lines.map { it.split('\t') }
                    .associate { it[0] to it[1] }
            }
        } catch (e: IOException) {
            throw IOException("Failed to load transcription data from $TRANSCRIPTION_RESOURCE_PATH", e)
        }
    }
}
