package com.marvinelsen.chinese.phonetics

import com.marvinelsen.chinese.phonetics.internal.syllable.Syllable
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.PinyinMarkSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.PinyinNumberSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.formatting.ZhuyinSyllableFormatter
import com.marvinelsen.chinese.phonetics.internal.syllable.parsing.PinyinNumberSyllableParser
import com.marvinelsen.chinese.phonetics.internal.syllable.parsing.ZhuyinSyllableParser
import io.kotest.core.spec.style.ShouldSpec
import io.kotest.datatest.withData
import io.kotest.matchers.shouldBe

class ChinesePhoneticsTest : ShouldSpec({
    context("from pinyin with tone numbers") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            "sheng1" to Syllable("sheng", Tone.FIRST),
            "zhi2" to Syllable("zhi", Tone.SECOND),
            "ka3" to Syllable("ka", Tone.THIRD),
            "yao4" to Syllable("yao", Tone.FOURTH),
            "me5" to Syllable("me", Tone.FIFTH),
            "Me5" to Syllable("me", Tone.FIFTH),
            "nv3" to Syllable("nü", Tone.THIRD),
            "nü3" to Syllable("nü", Tone.THIRD),
            "nu:3" to Syllable("nü", Tone.THIRD),
            "r5" to Syllable("r", Tone.FIFTH),
            "R5" to Syllable("r", Tone.FIFTH),
            "er2" to Syllable("er", Tone.SECOND),
            "Er2" to Syllable("er", Tone.SECOND),
        ) { (pinyinWithNumber, expectedSyllable) ->
            PinyinNumberSyllableParser.parse(pinyinWithNumber) shouldBe expectedSyllable
        }
    }
    context("from zhuyin") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            "ㄕㄥ" to Syllable("sheng", Tone.FIRST),
            "ㄓˊ" to Syllable("zhi", Tone.SECOND),
            "ㄎㄚˇ" to Syllable("ka", Tone.THIRD),
            "ㄧㄠˋ" to Syllable("yao", Tone.FOURTH),
            "ㄇㄜ˙" to Syllable("me", Tone.FIFTH),
            "˙ㄇㄜ" to Syllable("me", Tone.FIFTH),
            "ㄋㄩˇ" to Syllable("nü", Tone.THIRD),
        ) { (zhuyin, expectedSyllable) ->
            ZhuyinSyllableParser.parse(zhuyin) shouldBe expectedSyllable
        }
    }

    context("from invalid pinyin with tone numbers") {
    }

    context("from invalid zhuyin") {
    }

    context("format to zhuyin") {
        withData(
            nameFn = { "$it -> ${it.second}" },
            Syllable("sheng", Tone.FIRST) to "ㄕㄥ",
            Syllable("zhi", Tone.SECOND) to "ㄓˊ",
            Syllable("ka", Tone.THIRD) to "ㄎㄚˇ",
            Syllable("yao", Tone.FOURTH) to "ㄧㄠˋ",
            Syllable("me", Tone.FIFTH) to "˙ㄇㄜ",
            Syllable("nü", Tone.THIRD) to "ㄋㄩˇ",
            Syllable("r", Tone.FIFTH) to "˙ㄦ",
            Syllable("er", Tone.SECOND) to "ㄦˊ",
        ) { (syllable, expectedZhuyin) ->
            ZhuyinSyllableFormatter.format(syllable) shouldBe expectedZhuyin
        }
    }

    context("format to pinyin with tone numbers") {
        withData(
            nameFn = { "$it -> ${it.second}" },
            Syllable("sheng", Tone.FIRST) to "sheng1",
            Syllable("zhi", Tone.SECOND) to "zhi2",
            Syllable("ka", Tone.THIRD) to "ka3",
            Syllable("yao", Tone.FOURTH) to "yao4",
            Syllable("me", Tone.FIFTH) to "me5",
            Syllable("nü", Tone.THIRD) to "nü3",
            Syllable("r", Tone.FIFTH) to "r5",
            Syllable("er", Tone.SECOND) to "er2",
        ) { (syllable, expectedPinyinWithToneNumbers) ->
            PinyinNumberSyllableFormatter.format(syllable) shouldBe expectedPinyinWithToneNumbers
        }
    }

    context("format to pinyin with tone marks") {
        withData(
            nameFn = { "$it -> ${it.second}" },
            Syllable("sheng", Tone.FIRST) to "shēng",
            Syllable("zhi", Tone.SECOND) to "zhí",
            Syllable("ka", Tone.THIRD) to "kǎ",
            Syllable("yao", Tone.FOURTH) to "yào",
            Syllable("me", Tone.FIFTH) to "me",
            Syllable("zhui", Tone.FIRST) to "zhuī",
            Syllable("liu", Tone.FIRST) to "liū",
            Syllable("nü", Tone.THIRD) to "nǚ",
            Syllable("r", Tone.FIFTH) to "r",
            Syllable("er", Tone.SECOND) to "ér",
        ) { (syllable, expectedPinyinWithToneMarks) ->
            PinyinMarkSyllableFormatter.format(syllable) shouldBe expectedPinyinWithToneMarks
        }
    }

    context("format to zhuyin with invalid pinyin syllable") {
    }

    context("format to pinyin with tone diacritics with invalid pinyin syllable") {
    }

    context("format to pinyin with tone numbers with invalid pinyin syllable") {
    }
})
