package com.marvinelsen.chinese.phonetics

import com.marvinelsen.chinese.phonetics.exceptions.InvalidToneInputException
import io.kotest.assertions.throwables.shouldThrow
import io.kotest.core.spec.style.ShouldSpec
import io.kotest.datatest.withData
import io.kotest.matchers.nulls.shouldBeNull
import io.kotest.matchers.shouldBe

class ToneTest : ShouldSpec({
    context("convert correctly from digit") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            '1' to Tone.FIRST,
            '2' to Tone.SECOND,
            '3' to Tone.THIRD,
            '4' to Tone.FOURTH,
            '5' to Tone.FIFTH,
        ) { (digit, expectedTone) ->
            Tone.fromDigit(digit) shouldBe expectedTone
        }
    }

    context("convert correctly from Int") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            1 to Tone.FIRST,
            2 to Tone.SECOND,
            3 to Tone.THIRD,
            4 to Tone.FOURTH,
            5 to Tone.FIFTH,
        ) { (number, expectedTone) ->
            Tone.fromInt(number) shouldBe expectedTone
        }
    }

    context("convert correctly from Zhuyin tone mark") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            '¯' to Tone.FIRST,
            'ˊ' to Tone.SECOND,
            'ˇ' to Tone.THIRD,
            'ˋ' to Tone.FOURTH,
            '˙' to Tone.FIFTH,
        ) { (zhuyinToneMark, expectedTone) ->
            Tone.fromZhuyinTone(zhuyinToneMark) shouldBe expectedTone
        }
    }

    context("convert correctly from Pinyin tone mark") {
        withData(
            nameFn = { "'${it.first}' -> ${it.second}" },
            '\u0304' to Tone.FIRST,
            '\u0301' to Tone.SECOND,
            '\u030C' to Tone.THIRD,
            '\u0300' to Tone.FOURTH,
        ) { (pinyinToneMark, expectedTone) ->
            Tone.fromPinyinTone(pinyinToneMark) shouldBe expectedTone
        }
    }

    context("convert correctly to Int") {
        withData(
            nameFn = { "${it.first} -> '${it.second}'" },
            Tone.FIRST to 1,
            Tone.SECOND to 2,
            Tone.THIRD to 3,
            Tone.FOURTH to 4,
            Tone.FIFTH to 5,
        ) { (tone, expectedInteger) ->
            tone.toInt() shouldBe expectedInteger
        }
    }

    context("convert correctly to Zhuyin tone mark") {
        withData(
            nameFn = { "${it.first} -> '${it.second}'" },
            Tone.FIRST to "",
            Tone.SECOND to "ˊ",
            Tone.THIRD to "ˇ",
            Tone.FOURTH to "ˋ",
            Tone.FIFTH to "˙",
        ) { (tone, zhuyinTone) ->
            tone.toZhuyinTone() shouldBe zhuyinTone
        }
    }

    context("convert correctly to Pinyin tone mark") {
        withData(
            nameFn = { "${it.first} -> '${it.second}'" },
            Tone.FIRST to "\u0304",
            Tone.SECOND to "\u0301",
            Tone.THIRD to "\u030C",
            Tone.FOURTH to "\u0300",
            Tone.FIFTH to "",
        ) { (tone, pinyinTone) ->
            tone.toPinyinTone() shouldBe pinyinTone
        }
    }

    context("convert correctly to Pinyin number") {
        withData(
            nameFn = { "${it.first} -> '${it.second}'" },
            Tone.FIRST to "1",
            Tone.SECOND to "2",
            Tone.THIRD to "3",
            Tone.FOURTH to "4",
            Tone.FIFTH to "5",
        ) { (tone, pinyinNumber) ->
            tone.toPinyinNumber() shouldBe pinyinNumber
        }
    }

    context("return null when converting from invalid digit ") {
        withData(
            nameFn = { "'$it' -> null" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidDigit ->
            Tone.fromDigitOrNull(invalidDigit).shouldBeNull()
        }
    }

    context("return null when converting from invalid int ") {
        withData(
            nameFn = { "'$it' -> null" },
            0,
            6,
            -1,
            Int.MAX_VALUE,
            Int.MIN_VALUE,
        ) { invalidNumber ->
            Tone.fromIntOrNull(invalidNumber).shouldBeNull()
        }
    }

    context("return null when converting from invalid Zhuyin tone mark ") {
        withData(
            nameFn = { "'$it' -> null" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidZhuyinToneMark ->
            Tone.fromZhuyinToneOrNull(invalidZhuyinToneMark).shouldBeNull()
        }
    }

    context("return null when converting from invalid Pinyin tone mark ") {
        withData(
            nameFn = { "'$it' -> null" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidPinyinToneMark ->
            Tone.fromPinyinToneOrNull(invalidPinyinToneMark).shouldBeNull()
        }
    }

    context("throw exception when converting from invalid digit ") {
        withData(
            nameFn = { "'$it' -> throws exception" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidDigit ->
            shouldThrow<InvalidToneInputException> {
                Tone.fromDigit(invalidDigit)
            }
        }
    }

    context("throw exception when converting from invalid int ") {
        withData(
            nameFn = { "'$it' -> throws exception" },
            0,
            6,
            -1,
            Int.MAX_VALUE,
            Int.MIN_VALUE,
        ) { invalidNumber ->
            shouldThrow<InvalidToneInputException> {
                Tone.fromInt(invalidNumber)
            }
        }
    }

    context("throw exception when converting from invalid Zhuyin tone mark ") {
        withData(
            nameFn = { "'$it' -> throws exception" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidZhuyinToneMark ->
            shouldThrow<InvalidToneInputException> {
                Tone.fromZhuyinTone(invalidZhuyinToneMark)
            }
        }
    }

    context("throw exception when converting from invalid Pinyin tone mark ") {
        withData(
            nameFn = { "'$it' -> throws exception" },
            '0',
            '6',
            'a',
            'z',
            '$',
            '*',
        ) { invalidPinyinToneMark ->
            shouldThrow<InvalidToneInputException> {
                Tone.fromPinyinTone(invalidPinyinToneMark)
            }
        }
    }
})
